Waker

Struct Waker 

1.36.0 ยท Source
pub struct Waker { /* private fields */ }
A Waker is a handle for waking up a task by notifying its executor that it is ready to be run.

This handle encapsulates a RawWaker instance, which defines the executor-specific wakeup behavior.

The typical life of a Waker is that it is constructed by an executor, wrapped in a Context, then passed to Future::poll(). Then, if the future chooses to return Poll::Pending, it must also store the waker somehow and call Waker::wake() when the future should be polled again.

Implements Clone, Send, and Sync; therefore, a waker may be invoked from any thread, including ones not in any way managed by the executor. For example, this might be done to wake a future when a blocking function call completes on another thread.

Note that it is preferable to use waker.clone_from(&new_waker) instead of *waker = new_waker.clone(), as the former will avoid cloning the waker unnecessarily if the two wakers wake the same task.

Constructing a Waker from a RawWaker is unsafe. Implementing the Wake trait is a safe alternative that requires memory allocation.

Implementations

Source

impl Waker

1.36.0 ยท Source

pub fn wake(self)

Wakes up the task associated with this Waker.

As long as the executor keeps running and the task is not finished, it is guaranteed that each invocation of wake() (or wake_by_ref()) will be followed by at least one poll() of the task to which this Waker belongs, such that the call to wake() (or wake_by_ref()) happens-before the beginning of the invocation of poll(). This makes it possible to temporarily yield to other tasks while running potentially unbounded processing loops.

Note that the above implies that multiple wake-ups may be coalesced into a single poll() invocation by the executor.

Also note that yielding to competing tasks is not guaranteed: it is the executorโ€™s choice which task to run and the executor may choose to run the current task again.

1.36.0 ยท Source

pub fn wake_by_ref(&self)

Wakes up the task associated with this Waker without consuming the Waker.

This is similar to wake(), but may be slightly less efficient in the case where an owned Waker is available. This method should be preferred to calling waker.clone().wake().

1.36.0 ยท Source

pub fn will_wake(&self, other: &Waker) -> bool

Returns true if this Waker and another Waker would awake the same task.

This function works on a best-effort basis, and may return false even when the Wakers would awaken the same task. However, if this function returns true, it is guaranteed that the Wakers will awaken the same task.

This function is primarily used for optimization purposes โ€” for example, this typeโ€™s clone_from implementation uses it to avoid cloning the waker when they would wake the same task anyway.

1.83.0 (const: 1.83.0) ยท Source

pub const unsafe fn new( data: *const (), vtable: &'static RawWakerVTable, ) -> Waker

Creates a new Waker from the provided data pointer and vtable.

The data pointer can be used to store arbitrary data as required by the executor. This could be e.g. a type-erased pointer to an Arc that is associated with the task. The value of this pointer will get passed to all functions that are part of the vtable as the first parameter.

It is important to consider that the data pointer must point to a thread safe type such as an Arc.

The vtable customizes the behavior of a Waker. For each operation on the Waker, the associated function in the vtable will be called.

Safety

The behavior of the returned Waker is undefined if the contract defined in RawWakerVTableโ€™s documentation is not upheld.

(Authors wishing to avoid unsafe code may implement the Wake trait instead, at the cost of a required heap allocation.)

1.36.0 (const: 1.82.0) ยท Source

pub const unsafe fn from_raw(waker: RawWaker) -> Waker

Creates a new Waker from RawWaker.

Safety

The behavior of the returned Waker is undefined if the contract defined in RawWakerโ€™s and RawWakerVTableโ€™s documentation is not upheld.

(Authors wishing to avoid unsafe code may implement the Wake trait instead, at the cost of a required heap allocation.)

1.85.0 (const: 1.85.0) ยท Source

pub const fn noop() -> &'static Waker

Returns a reference to a Waker that does nothing when used.

This is mostly useful for writing tests that need a Context to poll some futures, but are not expecting those futures to wake the waker or do not need to do anything specific if it happens.

More generally, using Waker::noop() to poll a future means discarding the notification of when the future should be polled again. So it should only be used when such a notification will not be needed to make progress.

If an owned Waker is needed, clone() this one.

Examples
use std::future::Future;
use std::task;

let mut cx = task::Context::from_waker(task::Waker::noop());

let mut future = Box::pin(async { 10 });
assert_eq!(future.as_mut().poll(&mut cx), task::Poll::Ready(10));
1.83.0 ยท Source

pub fn data(&self) -> *const ()

Gets the data pointer used to create this Waker.

1.83.0 ยท Source

pub fn vtable(&self) -> &'static RawWakerVTable

Gets the vtable pointer used to create this Waker.

pub const fn from_fn_ptr(f: fn()) -> Waker

๐Ÿ”ฌThis is a nightly-only experimental API. (waker_from_fn_ptr #148457)

Constructs a Waker from a function pointer.